@@ -1,5 +1,5 @@
 char   netcpu_kstat10_id[]="\
-@(#)netcpu_kstat10.c (c) Copyright 2005-2007, Hewlett-Packard Company Version 2.4.3";
+@(#)netcpu_kstat10.c (c) Copyright 2005-2011, Hewlett-Packard Company Version 2.5.0";
 
 #if HAVE_CONFIG_H
 # include <config.h>
@@ -56,20 +56,24 @@ static cpu_time_counters_t corrected_cpu_counters[MAXCPUS];
 static void
 print_cpu_time_counters(char *name, int instance, cpu_time_counters_t *counters) 
 {
-  fprintf(where,"%s[%d]:\n",name,instance);
   fprintf(where,
-	  "\t idle %llu\n",counters[instance].idle);
-  fprintf(where,
-	  "\t user %llu\n",counters[instance].user);
-  fprintf(where,
-	  "\t kernel %llu\n",counters[instance].kernel);
-  fprintf(where,
-	  "\t interrupt %llu\n",counters[instance].interrupt);
+	  "%s[%d]:\n"
+	  "\t idle %llu\n"
+	  "\t user %llu\n"
+	  "\t kernel %llu\n"
+	  "\t interrupt %llu\n",
+	  name,instance,
+	  counters[instance].idle,
+	  counters[instance].user,
+	  counters[instance].kernel,
+	  counters[instance].interrupt);
 }
 
 void
 cpu_util_init(void) 
 {
+  kstat_t   *ksp;
+  int i;
   kc = kstat_open();
 
   if (kc == NULL) {
@@ -80,6 +84,36 @@ cpu_util_init(void)
     fflush(where);
     exit(-1);
   }
+
+  /* lets flesh-out a CPU instance number map since it seems that some
+     systems, not even those which are partitioned, can have
+     non-contiguous CPU numbers.  discovered "the hard way" on a
+     T5220. raj 20080804 */
+  i = 0;
+  for (ksp = kc->kc_chain, i = 0;
+       (ksp != NULL) && (i < MAXCPUS);
+       ksp = ksp->ks_next) {  
+    if ((strcmp(ksp->ks_module,"cpu") == 0) &&
+	(strcmp(ksp->ks_name,"sys") == 0)) {
+      if (debug) {
+	fprintf(where,"Mapping CPU instance %d to entry %d\n",
+		ksp->ks_instance,i);
+	fflush(where);
+      }
+      lib_cpu_map[i++] = ksp->ks_instance;
+    }
+  }  
+
+  if (MAXCPUS == i) {
+    fprintf(where,
+            "Sorry, this system has more CPUs (%d) than netperf can handle (%d).\n"
+            "Please alter MAXCPUS in netlib.h and recompile.\n",
+            i,
+            MAXCPUS);
+    fflush(where);
+    exit(1);
+  }
+
   return;
 }
 
@@ -101,8 +135,7 @@ print_unexpected_statistic_warning(char *who, char *what, char *why)
 {
   if (why) {
     fprintf(where,
-	    "WARNING! WARNING! WARNING! WARNING!\n");
-    fprintf(where,
+	    "WARNING! WARNING! WARNING! WARNING!\n"
 	    "%s found an unexpected %s statistic %.16s\n",
 	    who,
 	    why,
@@ -126,7 +159,7 @@ get_cpu_counters(int cpu_num, cpu_time_counters_t *counters)
   kstat_named_t *knp;
   int i;
 
-  ksp = kstat_lookup(kc, "cpu", cpu_num, "sys");
+  ksp = kstat_lookup(kc, "cpu", lib_cpu_map[cpu_num], "sys");
   if ((ksp) && (ksp->ks_type == KSTAT_TYPE_NAMED)) {
     /* happiness and joy, keep going */
     nkcid = kstat_read(kc, ksp, NULL);
@@ -151,6 +184,13 @@ get_cpu_counters(int cpu_num, cpu_time_counters_t *counters)
 	  found++;
 	  counters[cpu_num].kernel = knp->value.ui64;
 	}
+	else if (!strcmp("cpu_nsec_intr",knp->name)) {
+	  if (debug >= 2) {
+	    fprintf(where,
+		    "Found a cpu_nsec_intr but it doesn't do what we want\n");
+	    fflush(where);
+	  }
+	}
 	else if (strstr(knp->name,"nsec")) {
 	  /* finding another nsec here means Sun have changed
 	     something and we need to warn the user. raj 2005-01-28 */ 
@@ -162,8 +202,7 @@ get_cpu_counters(int cpu_num, cpu_time_counters_t *counters)
 
 	  /* might want to tell people about what we are skipping.
 	     however, only display other names debug >=2. raj
-	     2005-01-28
-	  */
+	     2005-01-28  */
 
 	  print_unexpected_statistic_warning("get_cpu_counters",
 					     knp->name,
@@ -194,8 +233,9 @@ get_cpu_counters(int cpu_num, cpu_time_counters_t *counters)
   else {
     /* the lookup failed or found the wrong type */
     fprintf(where,
-	    "get_cpu_counters: kstat_lookup failed for module 'cpu' instance %d name 'sys' and KSTAT_TYPE_NAMED: errno %d %s\n",
+	    "get_cpu_counters: kstat_lookup failed for module 'cpu' number %d instance %d name 'sys' and KSTAT_TYPE_NAMED: errno %d %s\n",
 	    cpu_num,
+	    lib_cpu_map[cpu_num],
 	    errno,
 	    strerror(errno));
     fflush(where);
@@ -212,7 +252,7 @@ get_interrupt_counters(int cpu_num, cpu_time_counters_t *counters)
   kstat_named_t *knp;
   int i;
 
-  ksp = kstat_lookup(kc, "cpu", cpu_num, "intrstat");
+  ksp = kstat_lookup(kc, "cpu", lib_cpu_map[cpu_num], "intrstat");
 
   counters[cpu_num].interrupt = 0;
   if ((ksp) && (ksp->ks_type == KSTAT_TYPE_NAMED)) {
@@ -266,8 +306,9 @@ get_interrupt_counters(int cpu_num, cpu_time_counters_t *counters)
   else {
     /* the lookup failed or found the wrong type */
     fprintf(where,
-	    "get_cpu_counters: kstat_lookup failed for module 'cpu' instance %d class 'intrstat' and KSTAT_TYPE_NAMED: errno %d %s\n",
+	    "get_cpu_counters: kstat_lookup failed for module 'cpu' %d instance %d class 'intrstat' and KSTAT_TYPE_NAMED: errno %d %s\n",
 	    cpu_num,
+	    lib_cpu_map[cpu_num],
 	    errno,
 	    strerror(errno));
     fflush(where);
@@ -332,11 +373,11 @@ calc_cpu_util_internal(float elapsed_time)
 
   lib_local_cpu_util = (float)0.0;
 
-  /* It is possible that the library measured a time other than */
-  /* the one that the user want for the cpu utilization */
-  /* calculations - for example, tests that were ended by */
-  /* watchdog timers such as the udp stream test. We let these */
-  /* tests tell up what the elapsed time should be. */
+  /* It is possible that the library measured a time other than the
+     one that the user want for the cpu utilization calculations - for
+     example, tests that were ended by watchdog timers such as the udp
+     stream test. We let these tests tell up what the elapsed time
+     should be. */
   
   if (elapsed_time != 0.0) {
     correction_factor = (float) 1.0 + 
@@ -374,7 +415,7 @@ calc_cpu_util_internal(float elapsed_time)
        more interrupt time than the sum of user, kernel and idle.
        that is a theoretical possibility I suppose, but for the
        time-being, one that we will blythly ignore, except perhaps for
-       a quick check. raj 2005-01-31 
+       a quick check. raj 2005-01-31
     */
     
     /* we ass-u-me that these counters will never wrap during a
@@ -408,19 +449,17 @@ calc_cpu_util_internal(float elapsed_time)
 	 the best way to get our tails out of here so let us just
 	 punt. raj 2005-01-31 */
       fprintf(where,
-	      "WARNING! WARNING! WARNING! WARNING! WARNING! \n");
-      fprintf(where,
-	      "calc_cpu_util_internal: more interrupt time than others combined!\n");
-      fprintf(where,
-	      "\tso CPU util cannot be estimated\n");
-      fprintf(where,
-	      "\t delta[%d].interrupt %llu\n",i,delta_cpu_counters[i].interrupt);
-      fprintf(where,
-	      "\t delta[%d].idle %llu\n",i,delta_cpu_counters[i].idle);
-      fprintf(where,
-	      "\t delta[%d].user %llu\n",i,delta_cpu_counters[i].user);
-      fprintf(where,
-	      "\t delta[%d].kernel %llu\n",i,delta_cpu_counters[i].kernel);
+	      "WARNING! WARNING! WARNING! WARNING! WARNING! \n"
+	      "calc_cpu_util_internal: more interrupt time than others combined!\n"
+	      "\tso CPU util cannot be estimated\n"
+	      "\t delta[%d].interrupt %llu\n"
+	      "\t delta[%d].idle %llu\n"
+	      "\t delta[%d].user %llu\n"
+	      "\t delta[%d].kernel %llu\n",
+	      i,delta_cpu_counters[i].interrupt,
+	      i,delta_cpu_counters[i].idle,
+	      i,delta_cpu_counters[i].user,
+	      i,delta_cpu_counters[i].kernel);
       fflush(where);
       
       lib_local_cpu_util = -1.0;
@@ -455,14 +494,13 @@ calc_cpu_util_internal(float elapsed_time)
 
     if (debug) {
       fprintf(where,
-	      "\tfraction_idle %llu interrupt_idle %llu\n",
+	      "\tfraction_idle %llu interrupt_idle %llu\n"
+	      "\tfraction_user %llu interrupt_user %llu\n"
+	      "\tfraction_kernel %llu interrupt_kernel %llu\n",
 	      fraction_idle,
-	      interrupt_idle);
-      fprintf(where,
-	      "\tfraction_user %llu interrupt_user %llu\n",
+	      interrupt_idle,
 	      fraction_user,
-	      interrupt_user);
-      fprintf(where,"\tfraction_kernel %llu interrupt_kernel %llu\n",
+	      interrupt_user,
 	      fraction_kernel,
 	      interrupt_kernel);
     }
@@ -484,9 +522,9 @@ calc_cpu_util_internal(float elapsed_time)
 			      corrected_cpu_counters);
     }
 
-    /* I was going to checkfor going less than zero, but since all the
-       calculations are in unsigned quantities that would seem to be a
-       triffle silly... raj 2005-01-28 */
+    /* I was going to check for going less than zero, but since all
+       the calculations are in unsigned quantities that would seem to
+       be a triffle silly... raj 2005-01-28 */
 
     /* ok, now we sum the numbers again, this time including interrupt
        */
@@ -528,21 +566,20 @@ calc_cpu_util_internal(float elapsed_time)
     /* and finally, what is our CPU utilization? */
     lib_local_per_cpu_util[i] = 100.0 - (((float)fraction_idle / 
 					  (float)CALC_ACCURACY) * 100.0);
+    lib_local_per_cpu_util[i] *= correction_factor;
     if (debug) {
       fprintf(where,
-	      "lib_local_per_cpu_util[%d] %g\n",
+	      "lib_local_per_cpu_util[%d] %g cf %f\n",
 	      i,
-	      lib_local_per_cpu_util[i]);
+	      lib_local_per_cpu_util[i],
+	      correction_factor);
     }
     lib_local_cpu_util += lib_local_per_cpu_util[i];
   }
   /* we want the average across all n processors */
   lib_local_cpu_util /= (float)lib_num_loc_cpus;
-  
-  lib_local_cpu_util *= correction_factor;
+
   return lib_local_cpu_util;
-
-
 }
 
 void
