@@ -15,18 +15,6 @@ char   netcpu_ntperf_id[]="\
 # endif
 #endif
 
-#if 0
-#include <limits.h>
-#include <sys/types.h>
-#include <fcntl.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <math.h>
-#include <string.h>
-#endif
-
-#include <assert.h>
-
 #include <process.h>
 #include <time.h>
 
@@ -77,7 +65,7 @@ typedef ULONG (__stdcall *NT_QUERY_SYSTEM_INFORMATION)(
 NT_QUERY_SYSTEM_INFORMATION NtQuerySystemInformation = NULL;
 
 
-static LARGE_INTEGER TickHz = {0,0};
+static LARGE_INTEGER TickHz = {{0,0}};
 
 _inline LARGE_INTEGER ReadPerformanceCounter(VOID)
 {
@@ -190,7 +178,7 @@ PerfObj *InitPerfCntrs()
     {
       //Lint
       status = GetLastError();
-      fprintf(stderr, "GetProcAddressFailed, status: %X\n", status);
+      fprintf(stderr, "GetProcAddressFailed, status: %lX\n", status);
       exit(1);
     }
   
@@ -240,7 +228,7 @@ void RestartPerfCntrs(PerfObj *PerfCntrs)
 					   (PCHAR)&PerfCntrs->EndInfo[0], sizeof(SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION)*MAXCPUS,
 					   &returnLength )) != 0) 
     {
-      fprintf(stderr, "NtQuery failed, status: %X\n", status);
+      fprintf(stderr, "NtQuery failed, status: %lX\n", status);
       exit(1);
     }
   
@@ -250,7 +238,7 @@ void RestartPerfCntrs(PerfObj *PerfCntrs)
   if ((returnLength % sizeof(SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION)) != 0)
     {
       fprintf(stderr, "NtQuery didn't return expected amount of data\n");
-      fprintf(stderr, "Expected a multiple of %i, returned %i\n",
+      fprintf(stderr, "Expected a multiple of %i, returned %lu\n",
 	      sizeof(SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION), returnLength);
       exit(1);
     }
@@ -259,7 +247,7 @@ void RestartPerfCntrs(PerfObj *PerfCntrs)
   if (returnNumCPUs != (int)SystemInfo.dwNumberOfProcessors)
     {
       fprintf(stderr, "NtQuery didn't return expected amount of data\n");
-      fprintf(stderr, "Expected data for %i CPUs, returned %i\n",
+      fprintf(stderr, "Expected data for %i CPUs, returned %lu\n",
 	      (int)SystemInfo.dwNumberOfProcessors, returnNumCPUs);
       exit(1);
     }
@@ -297,7 +285,7 @@ double ReportPerfCntrs(PerfObj *PerfCntrs)
 {
   double tot_CPU_Util;
   int i;
-  int duration;  // in 100 usecs
+  double duration;  // in milliseconds
   
   LARGE_INTEGER ActualDuration;
   
@@ -339,13 +327,13 @@ double ReportPerfCntrs(PerfObj *PerfCntrs)
   ActualDuration.QuadPart = PerfCntrs->EndTime.QuadPart - 
     PerfCntrs->StartTime.QuadPart;
   
-  // convert to 1/10 milliseconds (100 usec) 
+  // convert to 100 usec (1/10th milliseconds) timebase.
   ActualDuration.QuadPart = (ActualDuration.QuadPart*10000)/TickHz.QuadPart;
-  duration = ActualDuration.LowPart;
+  duration = (double)ActualDuration.QuadPart/10.0;  // duration in ms
   
   if (verbosity > 1)
     {
-      fprintf(where,"ActualDuration (ms): %d\n", duration/10);
+      fprintf(where,"ActualDuration (ms): %d\n", (int)duration);
     }
   
   if (verbosity > 1)
@@ -424,14 +412,14 @@ double ReportPerfCntrs(PerfObj *PerfCntrs)
       fprintf(where, "\n\n");
       
       fprintf(where, "Interrupt/Sec. %5.1f", 
-	      (double)DeltaInfo[MAXCPUS].InterruptCount*10000.0/(double)duration);
+	      (double)DeltaInfo[MAXCPUS].InterruptCount*1000.0/duration);
       
       if ((int)SystemInfo.dwNumberOfProcessors > 1)
 	{
 	  for (i=0; i < (int)SystemInfo.dwNumberOfProcessors; i++)
 	    {
 	      fprintf(where, "\t %5.1f", 
-		      (double)DeltaInfo[i].InterruptCount*10000.0/(double)duration);
+		      (double)DeltaInfo[i].InterruptCount*1000.0/duration);
 	    }
 	}
       fprintf(where, "\n\n");
