@@ -158,6 +158,10 @@
 #define         TCP_MAERTS_RESPONSE         95
 #define         TCP_MAERTS_RESULTS          96
 
+#define         DO_OMNI                     97
+#define         OMNI_RESPONSE               98
+#define         OMNI_RESULTS                99
+
 #define         DO_LWPSTR_STREAM           100
 #define         LWPSTR_STREAM_RESPONSE     110
 #define         LWPSTR_STREAM_RESULTS      120
@@ -178,6 +182,9 @@
 #define         TCP_CC_RESPONSE            301
 #define         TCP_CC_RESULTS             302
 
+/* The DNS_RR test has been removed from netperf but we leave these
+   here for historical purposes.  Those wanting to do DNS_RR tests
+   should use netperf4 instead. */
 #define         DO_DNS_RR                  400
 #define         DNS_RR_RESPONSE            401
 #define         DNS_RR_RESULTS             402
@@ -198,6 +205,29 @@
 #define         SCTP_RR_MANY_RESPONSE      531
 #define         SCTP_RR_MANY_RESULT        532
 
+#define         DO_SDP_STREAM              540
+#define         SDP_STREAM_RESPONSE        541
+#define         SDP_STREAM_RESULTS         542
+
+#define         DO_SDP_RR                  543
+#define         SDP_RR_RESPONSE            544
+#define         SDP_RR_RESULTS             545
+
+#define         DO_SDP_MAERTS              546
+#define         SDP_MAERTS_RESPONSE        547
+#define         SDP_MAERTS_RESULTS         548
+
+#define         DO_SDP_CRR                 549
+#define         SDP_CRR_RESPONSE           550
+#define         SDP_CRR_RESULTS            551
+
+#define         DO_SDP_CC                  552
+#define         SDP_CC_RESPONSE            553
+#define         SDP_CC_RESULTS             554
+
+#define         DO_SYSINFO                 600
+#define         SYSINFO_RESPONSE           601
+
 #if HAVE_INTTYPES_H
 # include <inttypes.h>
 #else
@@ -211,6 +241,20 @@ enum sock_buffer{
   RECV_BUFFER
 };
 
+/* some defines for security types, perhaps these would be better
+   elsewhere but for now here they are */
+
+#define NSEC_UNKNOWN  -1
+#define NSEC_DISABLED 0
+#define NSEC_PERMISSIVE  1
+#define NSEC_ENFORCING 2
+
+#define NSEC_TYPE_UNKNOWN -1
+#define NSEC_TYPE_SELINUX 1
+
+#define NETFW_UNKNOWN -1
+#define NETFW_IPTABLES 1
+
  /* some of the fields in these structures are going to be doubles and */
  /* such. so, we probably want to ensure that they will start on */
  /* "double" boundaries. this will break compatability to pre-2.1 */
@@ -326,7 +370,8 @@ extern void PrintWin32Error(FILE *stream, LPSTR text);
 		(((return_value) == SOCKET_ERROR) && \
 	     ((errno == EINTR) || \
 	      (errno == WSAECONNABORTED) || \
-	      (errno == WSAECONNRESET) ))
+	      (errno == WSAECONNRESET) || \
+	      (errno == ENOTSOCK) ))
 #define SOCKET_EADDRINUSE(return_value) \
 		(((return_value) == SOCKET_ERROR) && \
 	     ((errno == WSAEADDRINUSE) ))
@@ -404,8 +449,14 @@ extern char remote_data_port[10];
 extern char *local_data_address;
 extern char *remote_data_address;
 
+extern char *local_sysname, *remote_sysname;
+extern char *local_release, *remote_release;
+extern char *local_version, *remote_version;
+extern char *local_machine, *remote_machine;
+
 extern int local_data_family;
 extern int remote_data_family;
+extern int control_family;
 
 extern  union netperf_request_struct netperf_request;
 extern  union netperf_response_struct netperf_response;
@@ -413,11 +464,16 @@ extern  union netperf_response_struct netperf_response;
 extern  float    lib_local_cpu_util;
 extern  float    lib_elapsed;
 extern  float    lib_local_maxrate;
+extern  double   lib_local_peak_cpu_util;
+extern  int      lib_local_peak_cpu_id;
+extern  double   lib_remote_peak_cpu_util;
+extern  int      lib_remote_peak_cpu_id;
 
 extern  char    libfmt;
 
 extern  int     cpu_method;
 extern  int     lib_num_loc_cpus;
+extern  int     lib_num_rem_cpus;
 extern  SOCKET  server_sock;
 extern  int     times_up;
 extern  FILE    *where;
@@ -427,6 +483,9 @@ extern  float   lib_local_per_cpu_util[];
 extern  void    netlib_init();
 extern  int     netlib_get_page_size();
 extern  void    install_signal_catchers();
+extern  struct addrinfo *resolve_host(char hostname[], 
+				      char port[], 
+				      int af);
 extern  void    establish_control(char hostname[], 
 				  char port[], 
 				  int af,
@@ -438,7 +497,11 @@ extern  void    init_stat();
 extern  void    send_request();
 extern  void    recv_response();
 extern  void    send_response();
-extern  void    recv_request();
+extern  int     recv_request();
+extern  void    send_request_n(int n);  /* convert only the first N ints */
+extern  void    recv_response_n(int n); /* of the test-specific data via */
+extern  void    send_response_n(int n); /* htonl/ntonl as required */
+extern  int     recv_request_n(int n);
 extern  void    dump_request();
 extern  void    dump_addrinfo(FILE *dumploc, struct addrinfo *info,
 			      char *host, char *port, int family);
@@ -459,25 +522,45 @@ extern  void	retrieve_confident_values(float *elapsed_time,
 			  float *remote_cpu_utilization,
 			  float *local_service_demand,
 			  float *remote_service_demand);
+extern  double  get_result_confid();
+extern  double  get_loc_cpu_confid();
+extern  double  get_rem_cpu_confid();
 extern  void    display_confidence();
+extern  void    get_sock_buffer(SOCKET sd,
+				enum sock_buffer which,
+				int *effective_sizep);
 extern  void    set_sock_buffer(SOCKET sd,
 				enum sock_buffer which,
 				int requested_size,
 				int *effective_sizep);
 extern  char   *format_units();
 
+extern  void    get_remote_system_info();
+
 extern  char    *inet_ftos(int family);
 extern  char    *inet_ttos(int type);
 extern  char    *inet_ptos(int protocol);
+extern  char    *nsec_enabled_to_str(int enabled);
+extern  char    *nsec_type_to_str(int type);
 extern  double  ntohd(double net_double);
 extern  double  htond(double host_double);
 extern  int     inet_nton(int af, const void *src, char *dst, int cnt);
+extern  void    random_ip_address(struct addrinfo *res, int mask_len);
 extern  void    libmain();
 extern  double  calc_thruput(double units_received);
 extern  double  calc_thruput_interval(double units_received,double elapsed);
+extern  double  calc_thruput_omni(double units_received);
+extern  double  calc_thruput_interval_omni(double units_received,double elapsed);
 extern  float   calibrate_local_cpu(float local_cpu_rate);
 extern  float   calibrate_remote_cpu();
 extern  void    bind_to_specific_processor(int processor_affinity,int use_cpu_map);
+extern int      set_nonblock (SOCKET sock);
+extern char     *find_egress_interface(struct sockaddr *source, struct sockaddr *dest);
+extern char     *find_interface_slot(char *interface_name);
+extern void     find_interface_ids(char *interface_name, int *vendor, int *device, int *sub_vend, int *sub_dev);
+extern void     find_driver_info(char *ifname, char *driver, char *version, char *firmware, char *bus, int len);
+extern void     find_system_info(char **system_model, char **cpu_model, int *cpu_frequency);
+
 #ifndef WIN32
 
 /* WIN32 requires that at least one of the file sets to select be
@@ -488,9 +571,14 @@ extern int msec_sleep( int msecs );
 #endif  /* WIN32 */
 extern  float   calc_cpu_util(float elapsed_time);
 extern  float	calc_service_demand(double units_sent,
-			  float elapsed_time,
-			  float cpu_utilization,
-			  int num_cpus);
+				    float elapsed_time,
+				    float cpu_utilization,
+				    int num_cpus);
+/* this one determines the unit divisor based on libfmt */
+extern  float	calc_service_demand_fmt(double units_sent,
+					float elapsed_time,
+					float cpu_utilization,
+					int num_cpus);
 #if defined(__hpux)
 extern  void    catcher(int, siginfo_t *,void *);
 #else
@@ -509,16 +597,27 @@ extern  struct ring_elt *allocate_exs_buffer_ring();
 extern  struct sendfile_ring_elt *alloc_sendfile_buf_ring();
 #endif /* HAVE_SENDFILE */
 #ifdef WANT_DLPI
-extern  int     dl_connect(int fd, unsigned char *rem_addr, int rem_addr_len);
-extern  int     dl_bind(int fd, int sap, int mode, char *dlsap_ptr, int *dlsap_len);
+/* it seems that AIX in its finite wisdom has some bogus define in an
+   include file which defines "rem_addr" which then screws-up this extern
+   unless we change the names to protect the guilty. reported by Eric
+   Jones */
+extern int dl_connect(int fd, unsigned char *remote_addr, int remote_addr_len);
+extern int dl_bind(int fd, int sap, int mode, char *dlsap_ptr, int *dlsap_len);
 extern  int     dl_open(char devfile[], int ppa);
 #endif /* WANT_DLPI */
 extern  char    format_cpu_method(int method);
 extern unsigned int convert(char *string);
+extern unsigned int convert_timespec(char *string);
 
+#ifdef WANT_INTERVALS
+extern void start_itimer(unsigned int interval_len_msec);
+#endif
  /* these are all for the confidence interval stuff */
 extern double confidence;
-
+extern double result_confid;
+extern double loc_cpu_confid;
+extern double rem_cpu_confid;
+extern int lib_cpu_map[];
 #endif
 
 #ifdef WIN32
