@@ -1,6 +1,6 @@
 /*
  
-	   Copyright (C) 1993-2005 Hewlett-Packard Company
+	   Copyright (C) 1993-2011 Hewlett-Packard Company
                          ALL RIGHTS RESERVED.
  
   The enclosed software and documentation includes copyrighted works
@@ -42,11 +42,15 @@
  
 */
 char	netperf_id[]="\
-@(#)netperf.c (c) Copyright 1993-2004 Hewlett-Packard Company. Version 2.3";
+@(#)netperf.c (c) Copyright 1993-2011 Hewlett-Packard Company. Version 2.5.0";
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
-#if HAVE_STRINGS_H
+#ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif
 
@@ -81,8 +85,15 @@ char	netperf_id[]="\
 #include "nettest_dlpi.h"
 #endif /* WANT_DLPI */
 
+#ifdef WANT_SDP
+#include "nettest_sdp.h"
+#endif
+
+/* The DNS tests have been removed from netperf2. Those wanting to do
+   DNS_RR tests should use netperf4 instead. */
+
 #ifdef DO_DNS
-#include "nettest_dns.h"
+#error DNS tests have been removed from netperf. Use netperf4 instead
 #endif /* DO_DNS */
 
 #ifdef WANT_SCTP
@@ -102,7 +113,7 @@ main(int argc, char *argv[])
   
   /* Initialize the winsock lib ( version 2.2 ) */
   if ( WSAStartup(MAKEWORD(2,2), &wsa_data) == SOCKET_ERROR ){
-    printf("WSAStartup() failed : %d\n", GetLastError()) ;
+    printf("WSAStartup() failed : %lu\n", GetLastError()) ;
     return 1 ;
   }
 #endif /* WIN32 */
@@ -122,8 +133,10 @@ main(int argc, char *argv[])
   }
   
   
-  establish_control(host_name,test_port,address_family,
-		    local_host_name,local_test_port,local_address_family);
+  if (!no_control) {
+    establish_control(host_name,test_port,address_family,
+		      local_host_name,local_test_port,local_address_family);
+  }
   
   if (strcasecmp(test_name,"TCP_STREAM") == 0) {
     send_tcp_stream(host_name);
@@ -131,6 +144,9 @@ main(int argc, char *argv[])
   else if (strcasecmp(test_name,"TCP_MAERTS") == 0) {
     send_tcp_maerts(host_name);
   }
+  else if (strcasecmp(test_name,"TCP_MSS") == 0) {
+    send_tcp_mss(host_name);
+  }
 #ifdef HAVE_ICSC_EXS
   else if (strcasecmp(test_name,"EXS_TCP_STREAM") == 0) {
     send_exs_tcp_stream(host_name);
@@ -226,15 +242,37 @@ main(int argc, char *argv[])
     send_sctp_stream_1toMany(host_name);
   }
   else if (strcasecmp(test_name, "SCTP_RR_MANY") == 0) {
-    send_sctp_stream_1toMany(host_name);
+    send_sctp_rr_1toMany(host_name);
   }
 #endif
   
 #ifdef DO_DNS
   else if (strcasecmp(test_name,"DNS_RR") == 0) {
-    send_dns_rr(host_name);
+    fprintf(stderr,
+	  "DNS tests can now be found in netperf4.\n");
+    fflush(stderr);
+    exit(-1);
   }
 #endif /* DO_DNS */
+#ifdef WANT_SDP
+  else if (strcasecmp(test_name,"SDP_STREAM") == 0) {
+    send_sdp_stream(host_name);
+  }
+  else if (strcasecmp(test_name,"SDP_MAERTS") == 0) {
+    send_sdp_maerts(host_name);
+  }
+  else if (strcasecmp(test_name,"SDP_RR") == 0) {
+    send_sdp_rr(host_name);
+  }
+#endif /* WANT_SDP */
+#ifdef WANT_OMNI
+  else if (strcasecmp(test_name,"OMNI") == 0) {
+    send_omni(host_name);
+  }
+  else if (strcasecmp(test_name,"UUID") == 0) {
+    print_uuid(host_name);
+  }
+#endif
   else {
     printf("The test you requested is unknown to this netperf.\n");
     printf("Please verify that you have the correct test name, \n");
@@ -242,7 +280,9 @@ main(int argc, char *argv[])
     exit(1);
   }
   
-  shutdown_control();
+  if (!no_control) {
+    shutdown_control();
+  }
   
 #ifdef WIN32
   /* Cleanup the winsock lib */
