@@ -1,7 +1,7 @@
 #include "netperf_version.h"
 
 char	netsh_id[]="\
-@(#)netsh.c (c) Copyright 1993-2007 Hewlett-Packard Company. Version 2.4.3pre";
+@(#)netsh.c (c) Copyright 1993-2011 Hewlett-Packard Company. Version 2.5.0";
 
 
 /****************************************************************/
@@ -17,9 +17,9 @@ char	netsh_id[]="\
 #include <sys/types.h>
 #ifndef WIN32
 #include <unistd.h>
-#ifndef __VMS
+#if !defined(__VMS) && !defined(MSDOS)
 #include <sys/ipc.h>
-#endif /* __VMS */
+#endif /* __VMS/MSDOS */
 #endif /* WIN32 */
 #include <fcntl.h>
 #ifndef WIN32
@@ -34,10 +34,17 @@ char	netsh_id[]="\
 #include <sys/time.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include <netdb.h>
 #else
 #include <time.h>
 #include <winsock2.h>
+/* while it is unlikely that anyone running Windows 2000 or NT 4 is
+   going to be trying to compile this, if they are they will want to
+   define DONT_IPV6 in the sources file */
+#ifndef DONT_IPV6
+#include <ws2tcpip.h>
+#endif
 #define netperf_socklen_t socklen_t
 #endif
 
@@ -90,10 +97,11 @@ double atof(const char *);
 /*									*/
 /************************************************************************/
 
- /* Some of the args take optional parameters. Since we are using */
- /* getopt to parse the command line, we will tell getopt that they do */
- /* not take parms, and then look for them ourselves */
-#define GLOBAL_CMD_LINE_ARGS "A:a:b:B:CcdDf:F:H:hi:I:k:K:l:L:n:NO:o:P:p:rt:T:v:VW:w:46"
+/* Some of the args take optional parameters. Since we are using
+   getopt to parse the command line, we will tell getopt that they do
+   not take parms, and then look for them ourselves */
+
+#define GLOBAL_CMD_LINE_ARGS "A:a:b:B:CcdDf:F:H:hi:I:jk:K:l:L:n:NO:o:P:p:rSs:t:T:v:VW:w:46"
 
 /************************************************************************/
 /*									*/
@@ -115,6 +123,7 @@ extern int sys_nerr;
 
 /* some names and such                                                  */
 char	*program;		/* program invocation name		*/
+char    *command_line;          /* a copy of the entire command line    */
 char	username[BUFSIZ];	/* login name of user			*/
 char	cmd_file[BUFSIZ];	/* name of the commands file		*/
 
@@ -134,7 +143,9 @@ char    fill_file[BUFSIZ];
 int
   debug,			/* debugging level */
   print_headers,		/* do/don't display headers */
-  verbosity;		/* verbosity level */
+  verbosity,		/* verbosity level */
+  keep_histogram,
+  keep_statistics;
 
 /* When specified with -B, this will be displayed at the end of the line
    for output that does not include the test header.  mostly this is
@@ -172,13 +183,21 @@ int
   remote_recv_align,	/* alignment for remote receives	*/
   remote_send_align,	/* alignment for remote sends		*/
   remote_send_offset = 0,
-  remote_recv_offset = 0;
+  remote_recv_offset = 0,
+  remote_send_width = 0,
+  remote_recv_width = 0;
 
-#if defined(WANT_INTERVALS) || defined(WANT_DEMO)
+/* hoist above the if for omni */
 int
   interval_usecs,
   interval_wate,
-  interval_burst;
+  interval_burst,
+  remote_interval_usecs,
+  remote_interval_burst;
+
+int wait_time_secs;
+
+#if defined(WANT_INTERVALS) || defined(WANT_DEMO)
 
 int demo_mode;                    /* are we actually in demo mode? */
 double demo_interval = 1000000.0; /* what is the desired interval to
@@ -193,12 +212,10 @@ double demo_units = 0.0;          /* what is our current best guess as
 double units_this_tick;
 #endif
 
-#ifdef DIRTY
 int	loc_dirty_count;
 int	loc_clean_count;
 int	rem_dirty_count;
 int	rem_clean_count;
-#endif /* DIRTY */
 
  /* some of the vairables for confidence intervals... */
 
@@ -209,8 +226,8 @@ int  result_confidence_only = 0;
 
 double interval;
 
- /* stuff to control the "width" of the buffer rings for sending and */
- /* receiving data */
+ /* stuff to control the "width" of the buffer rings for sending and
+    receiving data */
 int	send_width;
 int     recv_width;
 
@@ -228,8 +245,11 @@ Usage: netserver [options] \n\
 \n\
 Options:\n\
     -h                Display this text\n\
+    -D                Do not daemonize\n\
     -d                Increase debugging output\n\
+    -f                Do not spawn chilren for each test, run serially\n\
     -L name,family    Use name to pick listen address and family for family\n\
+    -N                No debugging output, even if netperf asks\n\
     -p portnum        Listen for connect requests on portnum.\n\
     -4                Do IPv4\n\
     -6                Do IPv6\n\
@@ -260,6 +280,7 @@ Global options:\n\
     -i max,min        Specify the max and min number of iterations (15,1)\n\
     -I lvl[,intvl]    Specify confidence level (95 or 99) (99) \n\
                       and confidence interval in percentage (10)\n\
+    -j                Keep additional timing statistics\n\
     -l testlen        Specify test duration (>0 secs) (<0 bytes|trans)\n\
     -L name|ip,fam *  Specify the local ip|name and address family\n\
     -o send,recv      Set the local send,recv buffer offsets\n\
@@ -269,6 +290,8 @@ Global options:\n\
     -p port,lport*    Specify netserver port number and/or local port\n\
     -P 0|1            Don't/Do display test headers\n\
     -r                Allow confidence to be hit on result only\n\
+    -s seconds        Wait seconds between test setup and test start\n\
+    -S                Set SO_KEEPALIVE on the data connection\n\
     -t testname       Specify test to perform\n\
     -T lcpu,rcpu      Request netperf/netserver be bound to local/remote cpu\n\
     -v verbosity      Specify the verbosity level\n\
@@ -290,11 +313,11 @@ value. To set the second value it must be preceded with a comma or be a\n\
 comma-separated pair. This is to retain previous netperf behaviour.\n"; 
 
 
-/* This routine will return the two arguments to the calling routine. */
-/* If the second argument is not specified, and there is no comma, */
-/* then the value of the second argument will be the same as the */
-/* value of the first. If there is a comma, then the value of the */
-/* second argument will be the value of the second argument ;-) */
+/* This routine will return the two arguments to the calling routine.
+   If the second argument is not specified, and there is no comma,
+   then the value of the second argument will be the same as the value
+   of the first. If there is a comma, then the value of the second
+   argument will be the value of the second argument ;-) */
 void
 break_args(char *s, char *arg1, char *arg2)
 
@@ -315,25 +338,26 @@ break_args(char *s, char *arg1, char *arg2)
   while ((*arg1++ = *s++) != '\0');
 }
 
-/* break_args_explicit
+/* break_args_explicit_sep
 
    this routine is somewhat like break_args in that it will separate a
-   pair of comma-separated values.  however, if there is no comma,
-   this version will not ass-u-me that arg2 should be the same as
-   arg1. raj 2005-02-04 */
+   pair of values using the given separator.  however, if there is no
+   separator this version will not ass-u-me that arg2 should be the
+   same as arg1. raj 20101129 */
+
 void
-break_args_explicit(char *s, char *arg1, char *arg2)
+break_args_explicit_sep(char *s, int sep, char *arg1, char *arg2)
 
 {
   char *ns;
-  ns = strchr(s,',');
+  ns = strchr(s,sep);
   if (ns) {
-    /* there was a comma arg2 should be the second arg*/
+    /* there was a separator arg2 should be the second arg*/
     *ns++ = '\0';
     while ((*arg2++ = *ns++) != '\0');
   }
   else {
-    /* there was not a comma, so we should make sure that arg2 is \0
+    /* there was no separator, so we should make sure that arg2 is \0
        lest something become confused. raj 2005-02-04 */
     *arg2 = '\0';
   };
@@ -341,6 +365,18 @@ break_args_explicit(char *s, char *arg1, char *arg2)
 
 }
 
+/* break_args_explicit - now just a wrapper around a call to
+   break_args_explicit_sep passing-in a ',' as the separator. raj
+   20101129 */ 
+
+void
+break_args_explicit(char *s, char *arg1, char *arg2)
+
+{
+  break_args_explicit_sep(s, ',', arg1, arg2);
+}
+
+
 /* given a string with possible values for setting an address family,
    convert that into one of the AF_mumble values - AF_INET, AF_INET6,
    AF_UNSPEC as apropriate. the family_string is compared in a
@@ -369,77 +405,228 @@ parse_address_family(char family_string[])
       strstr(temp,"4")) {
     return(AF_INET);
   }
+#if defined(AF_RDS)
+  if (strstr(temp,"af_rds") ||
+      strstr(temp,"32")) {
+    return(AF_RDS);
+  }
+#endif
   if (strstr(temp,"unspec") ||
       strstr(temp,"0")) {
     return(AF_UNSPEC);
   }
   fprintf(where,
-	  "WARNING! %s not recognized as an address family, using AF_UNPSEC\n",
+	  "WARNING! %s not recognized as an address family, using AF_UNPSEC\n"
+	  "Are you sure netperf was configured for that address family?\n",
 	  family_string);
-  fprintf(where,
-	  "Are you sure netperf was configured for that address family?\n");
   fflush(where);
   return(AF_UNSPEC);
 }
 
+int
+parse_socket_type(char socket_string[]) {
+
+  char temp[10];
+
+  strncpy(temp,socket_string,10);
+
+  if (debug) {
+    fprintf(where,
+	    "Attempting to parse socket type from %s derived from %s\n",
+	    temp,
+	    socket_string);
+  }
+
+#ifdef SOCK_STREAM
+  if (strstr(temp,"stream"))
+    return SOCK_STREAM;
+#endif
+#ifdef SOCK_DGRAM
+  if (strstr(temp,"dgram"))
+    return SOCK_DGRAM;
+#endif
+  return NST_UNKN;
+
+}
+
+int
+parse_direction(char direction_string[])
+{
+  char arg1[BUFSIZ],arg2[BUFSIZ];
+  int left, right;
+
+  if (NULL == direction_string) {
+    return 0;
+  }
+
+  if (direction_string[0] == '\0') {
+    return 0;
+  }
+
+  /* allow someone to "or" break_args_explicit will split at the first
+     '|' in the string so we know that arg1 has no '|' in it and arg2
+     might */
+  break_args_explicit_sep(direction_string,'|',arg1,arg2);
+
+  /* at this point only arg2 could contain a '|' so recurse on that */
+  right = parse_direction(arg2);
+
+  /* now we parse the "left side" or arg1 */
+  if (arg1[0] == '\0') {
+    left = 0;
+  }
+  else if ((strcasecmp(arg1,"xmit") == 0) ||
+	   (strcasecmp(arg1,"send") == 0) ||
+	   (strcasecmp(arg1,"stream") == 0) ||
+	   (strcasecmp(arg1,"transmit") == 0)) {
+    left = NETPERF_XMIT;
+  }
+  else if ((strcasecmp(arg1,"recv") == 0) ||
+	   (strcasecmp(arg1,"receive") == 0) ||
+	   (strcasecmp(arg1,"maerts") == 0)) {
+    /* yes, another magic number... */
+    left =  NETPERF_RECV;
+  }
+  else  if (strcasecmp(arg1,"rr") == 0) {
+    left = NETPERF_XMIT|NETPERF_RECV;
+  }
+  else {
+    /* we now "ass-u-me" it is a number that can be parsed by strtol()
+     */
+    left = strtol(arg1,NULL,0);
+  }
+
+  return (left | right);
+
+}
+
+int
+parse_protocol(char protocol_string[]) 
+{
+  char temp[10];
+
+  strncpy(temp,protocol_string,10);
+
+  if (debug) {
+    fprintf(where,
+	    "Attempting to parse protocol from %s derived from %s\n",
+	    temp,
+	    protocol_string);
+  }
+  
+#ifdef IPPROTO_TCP
+  if (!strcasecmp(temp,"tcp")){
+    socket_type = SOCK_STREAM;
+    return IPPROTO_TCP;
+  }
+#endif
+
+#ifdef IPPROTO_UDP
+  if (!strcasecmp(temp,"udp")) {
+    socket_type = SOCK_DGRAM;
+    return IPPROTO_UDP;
+  }
+#endif
+#ifdef IPPROTO_SCTP
+  if (!strcasecmp(temp,"sctp")) {
+    /* it can be more than one socket type */
+    return IPPROTO_SCTP;
+  }
+#endif
+#ifdef IPPROTO_SDP
+  if (!strcasecmp(temp,"sdp")) {
+    socket_type = SOCK_STREAM;
+    return IPPROTO_SDP;
+  }
+#endif
+#if defined(IPPROTO_DCCP) && defined(SOCK_DCCP)
+  if (!strcasecmp(temp,"dccp")) {
+    socket_type = SOCK_DCCP;
+    return IPPROTO_DCCP;
+  }
+#endif
+#ifdef IPPROTO_UDPLITE
+  if (!strcasecmp(temp,"udplite")) {
+    socket_type = SOCK_DGRAM;
+    return IPPROTO_UDPLITE;
+  }
+#endif
+  return IPPROTO_IP;
+}
+
 
 void
 set_defaults()
 {
   
-  /* stuff to say where this test is going                              */
+  /* stuff to say where this test is going  */
   strcpy(host_name,"");	      /* remote host name or ip addr  */
   strcpy(local_host_name,""); /* we want it to be INADDR_ANY */
-  strcpy(test_name,"TCP_STREAM");	/* which test to run 		*/
-  strncpy(test_port,"12865",PORTBUFSIZE); /* where is the test waiting    */
+  strcpy(test_name,"TCP_STREAM");	/* which test to run  */
+  strncpy(test_port,"12865",PORTBUFSIZE); /* where is the test
+					     waiting  */
   strncpy(local_test_port,"0",PORTBUFSIZE);/* INPORT_ANY as it were */
   address_family = AF_UNSPEC;
   local_address_family = AF_UNSPEC;
 
-  /* output controlling variables                               */
-  debug			= 0;/* debugging level			*/
-  print_headers		= 1;/* do print test headers		*/
-  verbosity		= 1;/* verbosity level			*/
+  /* output controlling variables  */
+  debug			= 0;/* debugging  */
+  print_headers		= 1;/* do print test headers  */
+  verbosity		= 1;/* verbosity level  */
+  keep_histogram        = 0;
+  keep_statistics       = 0;
+
   /* cpu variables */
-  local_cpu_usage	= 0;/* measure local cpu		*/
-  remote_cpu_usage	= 0;/* what do you think ;-)		*/
+  local_cpu_usage	= 0;/* measure local cpu  */
+  remote_cpu_usage	= 0;/* what do you think ;-)  */
   
   local_cpu_rate	= (float)0.0;
   remote_cpu_rate	= (float)0.0;
   
-  /* the end-test conditions for the tests - either transactions, bytes,  */
-  /* or time. different vars used for clarity - space is cheap ;-)        */
-  test_time	= 10;	/* test ends by time			*/
-  test_bytes	= 0;	/* test ends on byte count		*/
-  test_trans	= 0;	/* test ends on tran count		*/
+  /* the end-test conditions for the tests - either transactions,
+     bytes, or time. different vars used for clarity - space is cheap
+     ;-)  */
+  test_time	= 10;	/* test ends by time  */
+  test_bytes	= 0;	/* test ends on byte count  */
+  test_trans	= 0;	/* test ends on tran count  */
   
-  /* the alignment conditions for the tests				*/
-  local_recv_align	= 8;	/* alignment for local receives	*/
-  local_send_align	= 8;	/* alignment for local sends	*/
+  /* the alignment conditions for the tests  */
+  local_recv_align	= 8;	/* alignment for local receives */
+  local_send_align	= 8;	/* alignment for local sends  */
   remote_recv_align	= 8;	/* alignment for remote receives*/
-  remote_send_align	= 8;	/* alignment for remote sends	*/
+  remote_send_align	= 8;	/* alignment for remote sends  */
   
-#ifdef WANT_INTERVALS
-  /* rate controlling stuff */
+  /* rate controlling stuff, taken out of the #ifdef for omni */
   interval_usecs  = 0;
-  interval_wate   = 1;
+  interval_wate   = 0;
   interval_burst  = 0;
-#endif /* WANT_INTERVALS */
-  
+  remote_interval_usecs = 0;
+  remote_interval_burst = 0;
+
+/* wait time between control/data connection establishment and start
+   of data traffic  */
+  wait_time_secs = 0;
+
 #ifdef DIRTY
   /* dirty and clean cache stuff */
   loc_dirty_count = 0;
   loc_clean_count = 0;
   rem_dirty_count = 0;
   rem_clean_count = 0;
+#else
+  loc_dirty_count = -1;
+  loc_clean_count = -1;
+  rem_dirty_count = -1;
+  rem_clean_count = -1;
 #endif /* DIRTY */
 
- /* some of the vairables for confidence intervals... */
+ /* some of the vairables for confidence intervals... defaults will be
+    set as apropriate in the parsing of the command line  */
 
-  confidence_level = 99;
+  confidence_level = 0;
   iteration_min = 1;
   iteration_max = 1;
-  interval = 0.05; /* five percent? */
+  interval = 0.0;
 
   no_control = 0;
   strcpy(fill_file,"");
@@ -449,15 +636,31 @@ set_defaults()
 void
 print_netserver_usage()
 {
-  fwrite(netserver_usage, sizeof(char), strlen(netserver_usage), stderr);
+  fprintf(stderr, "%s", netserver_usage);
 }
 
 
 void
 print_netperf_usage()
 {
-  fwrite(netperf_usage1, sizeof(char), strlen(netperf_usage1),  stderr);
-  fwrite(netperf_usage2, sizeof(char), strlen(netperf_usage2),  stderr);
+  fprintf(stderr, "%s%s", netperf_usage1, netperf_usage2);
+}
+
+/* convert the specified string to upper case if we know how */
+static void
+convert_to_upper(char *source) 
+{
+#if defined(HAVE_TOUPPER)
+  int i,length;
+
+  length = strlen(source);
+
+  for (i=0; i < length; i++) {
+    source[i] = toupper(source[i]);
+  }
+#endif
+  return;
+
 }
 
 void
@@ -465,7 +668,10 @@ scan_cmd_line(int argc, char *argv[])
 {
   extern int	optind;           /* index of first unused arg 	*/
   extern char	*optarg;	  /* pointer to option string	*/
-  
+
+  int           cmnd_len;
+  char          *p;
+
   int		c;
   
   char	arg1[BUFSIZ],  /* argument holders		*/
@@ -473,16 +679,38 @@ scan_cmd_line(int argc, char *argv[])
   
   program = (char *)malloc(strlen(argv[0]) + 1);
   if (program == NULL) {
-    printf("malloc(%d) failed!\n", strlen(argv[0]) + 1);
-    exit(1);
+    printf("malloc() to store program name failed!\n");
+    exit(-1);
   }
   strcpy(program, argv[0]);
-  
-  /* Go through all the command line arguments and break them */
-  /* out. For those options that take two parms, specifying only */
-  /* the first will set both to that value. Specifying only the */
-  /* second will leave the first untouched. To change only the */
-  /* first, use the form first, (see the routine break_args.. */
+
+  /* brute force, but effective */
+  command_line = NULL;
+  cmnd_len = 0;
+  for (c = 0; c < argc; c++) {
+    cmnd_len += strlen(argv[c]);
+  }
+  cmnd_len += argc;  /* forget thee not the spaces */
+  command_line = malloc(cmnd_len+1);
+
+  if (command_line == NULL) {
+    printf("malloc(%d) failed!\n",cmnd_len);
+    exit(-1);
+  }
+  p = command_line;
+  for (c = 0; c < argc; c++) {
+    memcpy(p,argv[c],strlen(argv[c]));
+    p += strlen(argv[c]);
+    *p = ' ';
+    p += 1;
+  }
+  *--p = 0;
+
+  /* Go through all the command line arguments and break them out. For
+     those options that take two parms, specifying only the first will
+     set both to that value. Specifying only the second will leave the
+     first untouched. To change only the first, use the form first,
+     (see the routine break_args.. */
   
   while ((c= getopt(argc, argv, GLOBAL_CMD_LINE_ARGS)) != EOF) {
     switch (c) {
@@ -509,9 +737,8 @@ scan_cmd_line(int argc, char *argv[])
 	remote_recv_align = convert(arg2);
       break;
     case 'c':
-      /* measure local cpu usage please. the user */
-      /* may have specified the cpu rate as an */
-      /* optional parm */
+      /* measure local cpu usage please. the user may have specified
+         the cpu rate as an optional parm */
       if (argv[optind] && isdigit((unsigned char)argv[optind][0])){
 	/* there was an optional parm */
 	local_cpu_rate = (float)atof(argv[optind]);
@@ -530,6 +757,9 @@ scan_cmd_line(int argc, char *argv[])
       break;
     case 'd':
       debug++;
+#ifdef MSDOS
+      dbug_init();
+#endif
       break;
     case 'D':
 #if (defined WANT_DEMO)
@@ -546,9 +776,9 @@ scan_cmd_line(int argc, char *argv[])
 	}
       }
 #else 
-      printf("Sorry, Demo Mode not configured into this netperf.\n");
-      printf("please consider reconfiguring netperf with\n");
-      printf("--enable-demo=yes and recompiling\n");
+      printf("Sorry, Demo Mode not configured into this netperf.\n"
+	     "Please consider reconfiguring netperf with\n"
+	     "--enable-demo=yes and recompiling\n");
 #endif 
       break;
     case 'f':
@@ -577,6 +807,8 @@ scan_cmd_line(int argc, char *argv[])
       /* limit maximum to 30 iterations */
       if (iteration_max > 30) iteration_max = 30;
       if (iteration_min > 30) iteration_min = 30;
+      if (confidence_level == 0) confidence_level = 99;
+      if (interval == 0.0) interval = 0.05; /* five percent */
       break;
     case 'I':
       /* set the confidence level (95 or 99) and width */
@@ -586,10 +818,12 @@ scan_cmd_line(int argc, char *argv[])
       }
       if((confidence_level != 95) && (confidence_level != 99)){
 	printf("Only 95%% and 99%% confidence level is supported\n");
-	exit(1);
+	exit(-1);
       }
       if (arg2[0] ) {
-	interval = (double) convert(arg2)/100;
+	/* it doesn't make much sense to use convert() here so just
+	   use strtod() instead. raj 2007-10-24 */
+	interval = strtod(arg2,NULL)/100.0;
       }
       /* make sure that iteration_min and iteration_max are at least
 	 at a reasonable default value.  if a -i option has previously
@@ -597,7 +831,13 @@ scan_cmd_line(int argc, char *argv[])
 	 against 1 */ 
       if (iteration_min == 1) iteration_min = 3;
       if (iteration_max == 1) iteration_max = 10;
-
+      /* make sure that the interval is set if it isn't at its default
+	 value */
+      if (interval == 0.0) interval = 0.05; /* five percent */
+      break;
+    case 'j':
+      keep_histogram = 1;
+      keep_statistics = 1;
       break;
     case 'k':
       /* local dirty and clean counts */
@@ -660,13 +900,24 @@ scan_cmd_line(int argc, char *argv[])
 	 meaningful if cpu util is enabled */
       result_confidence_only = 1;
       break;
+    case 'S':
+      /* the user wishes us to set SO_KEEPALIVE */
+      want_keepalive = 1;
+      break;
+    case 's':
+      /* the user wishes us to sleep/pause some length of time before
+	 actually starting the test */ 
+      wait_time_secs = convert(optarg);
+      break;
     case 't':
-      /* set the test name */
-      strcpy(test_name,optarg);
+      /* set the test name and shift it to upper case so we don't have
+	 to worry about compares on things like substrings */
+      strncpy(test_name,optarg,sizeof(test_name)-1);
+      convert_to_upper(test_name);
       break;
     case 'T':
-      /* We want to set the processor on which netserver or netperf */
-      /* will run */
+      /* We want to set the processor on which netserver or netperf
+         will run */
       break_args(optarg,arg1,arg2);
       if (arg1[0]) {
 	local_proc_affinity = convert(arg1);
@@ -678,8 +929,8 @@ scan_cmd_line(int argc, char *argv[])
       cpu_binding_requested = 1;
       break;
     case 'W':
-      /* set the "width" of the user space data buffer ring. This will */
-      /* be the number of send_size buffers malloc'd in the tests */  
+      /* set the "width" of the user space data buffer ring. This will
+	 be the number of send_size buffers malloc'd in the tests */  
       break_args(optarg,arg1,arg2);
       if (arg1[0]) 
 	send_width = convert(arg1);
@@ -733,27 +984,56 @@ scan_cmd_line(int argc, char *argv[])
 	local_address_family = parse_address_family(arg2);
       break;
     case 'w':
-      /* We want to send requests at a certain wate. */
-      /* Remember that there are 1000000 usecs in a */
-      /* second, and that the packet rate is */
-      /* expressed in packets per millisecond. */
+      /* We want to send requests at a certain wate.  Remember that
+         there are 1000000 usecs in a second, and that the packet rate
+         is expressed in packets per millisecond. shuffle the #ifdef
+         around a bit to deal with both netperf and netserver possibly
+         doing intervals with omni tests */
+      break_args_explicit(optarg,arg1,arg2);
+      if (arg1[0]) {
 #ifdef WANT_INTERVALS
-      interval_usecs = convert_timespec(optarg);
-      interval_wate  = interval_usecs / 1000;
+	interval_usecs = convert_timespec(arg1);
+	interval_wate  = interval_usecs / 1000;
 #else
-      fprintf(where,
-	      "Packet rate control is not compiled in.\n");
+	fprintf(where,
+		"Packet rate control is not compiled in.\n");
 #endif
+      }
+      if (arg2[0]) {
+	/* we pass usecs to the remote and let it deal to cover both
+	   intervals and spin methods. if he wasn't intervals enabled
+	   he will return a suitable value back to us */
+	remote_interval_usecs = convert_timespec(arg2);
+      }
       break;
     case 'b':
       /* we want to have a burst so many packets per */
       /* interval. */
+      break_args_explicit(optarg,arg1,arg2);
+      if (arg1[0]) {
 #ifdef WANT_INTERVALS
-      interval_burst = convert(optarg);
+	interval_burst = convert(arg1);
+	/* set a default in case the user does not include the -w
+	   option */
+	if (interval_usecs == 0) {
+	  interval_wate = 1;
+	  interval_usecs = 1000;
+	}
 #else
-      fprintf(where,
-	      "Packet burst size is not compiled in. \n");
+	fprintf(where,
+		"Packet burst size is not compiled in. \n");
 #endif /* WANT_INTERVALS */
+      }
+      /* there is no ifdef here because we are sending this value to
+	 the remote, which may or may not have been compiled for
+	 intervals and we don't have a way of knowing on this side
+	 until we try */
+      if (arg2[0]) {
+	remote_interval_burst = convert(arg2);
+	if (remote_interval_usecs == 0) {
+	  remote_interval_usecs = 1000;
+	}
+      }
       break;
     case 'B':
       result_brand = malloc(strlen(optarg)+1);
@@ -786,10 +1066,14 @@ scan_cmd_line(int argc, char *argv[])
   }
   /* ok, what should our default hostname and local binding info be?
    */
+
   if ('\0' == host_name[0]) {
     /* host_name was not set */
     switch (address_family) {
     case AF_INET:
+#if defined(AF_RDS)
+    case AF_RDS:
+#endif
       strcpy(host_name,"localhost");
       break;
     case AF_UNSPEC:
@@ -798,6 +1082,9 @@ scan_cmd_line(int argc, char *argv[])
       switch (local_address_family) {
       case AF_INET:
       case AF_UNSPEC:
+#if defined(AF_RDS)
+      case AF_RDS:
+#endif
 	strcpy(host_name,"localhost");
 	break;
 #if defined(AF_INET6)
@@ -821,19 +1108,38 @@ scan_cmd_line(int argc, char *argv[])
 	     address_family);
       exit(-1);
     }
+  } else {
+    /* resolve the hostname and pull the address family from the
+       addrinfo */
+    struct addrinfo *ai;
+
+    ai = resolve_host(host_name, NULL, address_family);
+    if (!ai) {
+      printf("Netperf could not resolve %s as a host name\n", host_name);
+      exit(-1);
+    }
+    address_family = ai->ai_family;
+    freeaddrinfo(ai);
   }
-  
+
+
   /* now, having established the name to which the control will
      connect, from what should it come? */
   if ('\0' == local_host_name[0]) {
     switch (local_address_family) {
     case AF_INET:
+#if defined(AF_RDS)
+    case AF_RDS:
+#endif
       strcpy(local_host_name,"0.0.0.0");
       break;
     case AF_UNSPEC:
       switch (address_family) {
       case AF_INET:
       case AF_UNSPEC:
+#if defined(AF_RDS)
+    case AF_RDS:
+#endif
 	strcpy(local_host_name,"0.0.0.0");
 	break;
 #if defined(AF_INET6)
@@ -872,28 +1178,37 @@ scan_cmd_line(int argc, char *argv[])
   }
 
   /* parsing test-specific options used to be conditional on there
-    being a "--" in the option stream.  however, some of the tests
-    have other initialization happening in their "scan" routines so we
-    want to call them regardless. raj 2005-02-08 */
-    if ((strcasecmp(test_name,"TCP_STREAM") == 0) ||
+     being a "--" in the option stream.  however, some of the tests
+     have other initialization happening in their "scan" routines so
+     we want to call them regardless. raj 2005-02-08 */
+  /* while the parsing of the command line will upshift the test name,
+     since we don't know that there will always be a way to do so? we
+     will retain for now the strcasecmp calls rather than switch to
+     strcmp. raj 20101220 */
+    if (
+#ifndef WANT_MIGRATION
+	(strcasecmp(test_name,"TCP_STREAM") == 0) ||
+	(strcasecmp(test_name,"TCP_MAERTS") == 0) ||
+	(strcasecmp(test_name,"TCP_RR") == 0) ||
+	(strcasecmp(test_name,"TCP_CRR") == 0) ||
+	(strcasecmp(test_name,"UDP_STREAM") == 0) ||
+	(strcasecmp(test_name,"UDP_RR") == 0) ||
+#endif
 #ifdef HAVE_ICSC_EXS
-    (strcasecmp(test_name,"EXS_TCP_STREAM") == 0) ||
+	(strcasecmp(test_name,"EXS_TCP_STREAM") == 0) ||
 #endif /* HAVE_ICSC_EXS */ 
 #ifdef HAVE_SENDFILE
 	(strcasecmp(test_name,"TCP_SENDFILE") == 0) ||
 #endif /* HAVE_SENDFILE */
-	(strcasecmp(test_name,"TCP_MAERTS") == 0) ||
-	(strcasecmp(test_name,"TCP_RR") == 0) ||
-	(strcasecmp(test_name,"TCP_CRR") == 0) ||
 	(strcasecmp(test_name,"TCP_CC") == 0) ||
+	(strcasecmp(test_name,"TCP_MSS") == 0) ||
 #ifdef DO_1644
 	(strcasecmp(test_name,"TCP_TRR") == 0) ||
 #endif /* DO_1644 */
 #ifdef DO_NBRR
 	(strcasecmp(test_name,"TCP_TRR") == 0) ||
 #endif /* DO_NBRR */
-	(strcasecmp(test_name,"UDP_STREAM") == 0) ||
-	(strcasecmp(test_name,"UDP_RR") == 0))
+	(0))
       {
 	scan_sockets_args(argc, argv);
       }
@@ -946,34 +1261,52 @@ scan_cmd_line(int argc, char *argv[])
 	scan_sdp_args(argc, argv);
       }
 #endif
-    
+
+#ifdef WANT_OMNI
+    else if ((strcasecmp(test_name,"OMNI") == 0) ||
+#ifdef WANT_MIGRATION
+	     (strcasecmp(test_name,"TCP_STREAM") == 0) ||
+	     (strcasecmp(test_name,"TCP_MAERTS") == 0) ||
+	     (strcasecmp(test_name,"TCP_RR") == 0) ||
+	     (strcasecmp(test_name,"TCP_CRR") == 0) ||
+	     (strcasecmp(test_name,"UDP_STREAM") == 0) ||
+	     (strcasecmp(test_name,"UDP_RR") == 0) ||
+#endif
+	     (strcasecmp(test_name,"UUID") == 0)) {
+      scan_omni_args(argc, argv);
+    }
+#endif
+
     /* what is our default value for the output units?  if the test
        name contains "RR" or "rr" or "Rr" or "rR" then the default is
-       'x' for transactions. otherwise it is 'm' for megabits
-       (10^6) */
+       'x' for transactions. otherwise it is 'm' for megabits (10^6)
+       however...  if this is an "omni" test then we want to defer
+       this decision to scan_omni_args */
 
-    if ('?' == libfmt) {
-      /* we use a series of strstr's here because not everyone has
-	 strcasestr and I don't feel like up or downshifting text */
-      if ((strstr(test_name,"RR")) ||
-	  (strstr(test_name,"rr")) ||
-	  (strstr(test_name,"Rr")) ||
-	  (strstr(test_name,"rR"))) {
-	libfmt = 'x';
+    if (strcasecmp(test_name,"omni")) {
+      if ('?' == libfmt) {
+	/* we use a series of strstr's here because not everyone has
+	   strcasestr and I don't feel like up or downshifting text */
+	if ((strstr(test_name,"RR")) ||
+	    (strstr(test_name,"rr")) ||
+	    (strstr(test_name,"Rr")) ||
+	    (strstr(test_name,"rR"))) {
+	  libfmt = 'x';
+	}
+	else {
+	  libfmt = 'm';
+	}
       }
-      else {
-	libfmt = 'm';
-      }
-    }
-    else if ('x' == libfmt) {
-      /* now, a format of 'x' makes no sense for anything other than
-	 an RR test. if someone has been silly enough to try to set
-	 that, we will reset it silently to default - namely 'm' */
-      if ((strstr(test_name,"RR") == NULL) &&
-	  (strstr(test_name,"rr") == NULL) &&
-	  (strstr(test_name,"Rr") == NULL) &&
-	  (strstr(test_name,"rR") == NULL)) {
-	libfmt = 'm';
+      else if ('x' == libfmt) {
+	/* now, a format of 'x' makes no sense for anything other than
+	   an RR test. if someone has been silly enough to try to set
+	   that, we will reset it silently to default - namely 'm' */
+	if ((strstr(test_name,"RR") == NULL) &&
+	    (strstr(test_name,"rr") == NULL) &&
+	    (strstr(test_name,"Rr") == NULL) &&
+	    (strstr(test_name,"rR") == NULL)) {
+	  libfmt = 'm';
+	}
       }
     }
 }
